@@ -56,6 +56,8 @@ class BrailleGenerator(generator.Generator):
     SKIP_CONTEXT_ROLES = (pyatspi.ROLE_MENU,
                           pyatspi.ROLE_MENU_BAR,
                           pyatspi.ROLE_PAGE_TAB_LIST,
+                          pyatspi.ROLE_REDUNDANT_OBJECT,
+                          pyatspi.ROLE_UNKNOWN,
                           pyatspi.ROLE_COMBO_BOX)
 
     def __init__(self, script):
@@ -72,6 +74,20 @@ class BrailleGenerator(generator.Generator):
         globalsDict['Link'] = braille.Link
         globalsDict['asString'] = self.asString
 
+    def _isCandidateFocusedRegion(self, obj, region):
+        if not isinstance(region, (braille.Component, braille.Text)):
+            return False
+
+        try:
+            sameRole = obj.getRole() == region.accessible.getRole()
+            sameName = obj.name == region.accessible.name
+        except:
+            msg = 'ERROR: Could not get names, roles for %s, %s' % (obj, region.accessible)
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        return sameRole and sameName
+
     def generateBraille(self, obj, **args):
         if not _settingsManager.getSetting('enableBraille') \
            and not _settingsManager.getSetting('enableBrailleMonitor'):
@@ -96,13 +112,13 @@ class BrailleGenerator(generator.Generator):
             focusedRegion = result[0]
         except:
             focusedRegion = None
+        try:
+            role = obj.getRole()
+        except:
+            role = None
         for region in result:
-            try:
-                role = obj.getRole()
-            except:
-                role = None
             if isinstance(region, (braille.Component, braille.Text)) \
-               and region.accessible == obj:
+               and self._script.utilities.isSameObject(region.accessible, obj, True):
                 focusedRegion = region
                 break
             elif isinstance(region, braille.Text) \
@@ -115,6 +131,12 @@ class BrailleGenerator(generator.Generator):
                  and region.accessible.parent == obj:
                 focusedRegion = region
                 break
+        else:
+            candidates = list(filter(lambda x: self._isCandidateFocusedRegion(obj, x), result))
+            msg = 'INFO: Could not determine focused region. Candidates: %i' % len(candidates)
+            debug.println(debug.LEVEL_INFO, msg)
+            if len(candidates) == 1:
+                focusedRegion = candidates[0]
 
         return [result, focusedRegion]
 
@@ -140,6 +162,7 @@ class BrailleGenerator(generator.Generator):
         verbosityLevel = _settingsManager.getSetting('brailleVerbosityLevel')
 
         doNotPresent = [pyatspi.ROLE_UNKNOWN,
+                        pyatspi.ROLE_REDUNDANT_OBJECT,
                         pyatspi.ROLE_FILLER,
                         pyatspi.ROLE_EXTENDED,
                         pyatspi.ROLE_LINK]
@@ -332,7 +355,7 @@ class BrailleGenerator(generator.Generator):
         isWidget = lambda x: x and x.getRole() in widgetRoles
         result = []
         if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
-            widgets = pyatspi.findAllDescendants(obj, isWidget)
+            widgets = self._script.utilities.findAllDescendants(obj, isWidget)
             for widget in widgets:
                 result.extend(self.generate(widget, includeContext=False))
                 result.append(braille.Region(" "))
@@ -411,24 +434,6 @@ class BrailleGenerator(generator.Generator):
             self._restoreRole(oldRole, args)
         return result
 
-    def _generateComboBoxTextObj(self, obj, **args):
-        """For a combo box, we check to see if the text is editable. If so,
-        then we want to show the text attributes (such as selection --
-        see bug 496846 for more details).  This will return an array
-        containing a single object, which is the accessible for the
-        text object. Note that this is different from the rest of the
-        generators, which all return an array of strings.  Yes, this
-        is a hack.
-        """
-        result = []
-        textObj = None
-        for child in obj:
-            if child and child.getRole() == pyatspi.ROLE_TEXT:
-                textObj = child
-        if textObj and textObj.getState().contains(pyatspi.STATE_EDITABLE):
-            result.append(textObj)
-        return result
-
     def _generateIncludeContext(self, obj, **args):
         """Returns True or False to indicate whether context should be
         included or not.
