@@ -591,9 +591,14 @@ class Chat:
             return
 
         text = ""
-        if _settingsManager.getSetting('chatSpeakRoomName') and chatRoomName:
-            text = message.CHAT_MESSAGE_FROM_ROOM % chatRoomName
-        text = self._script.utilities.appendString(text, message)
+        if chatRoomName and \
+           _settingsManager.getAppSetting(self._script.app, 'chatSpeakRoomName'):
+            text = messages.CHAT_MESSAGE_FROM_ROOM % chatRoomName
+
+        if not settings.presentChatRoomLast:
+            text = self._script.utilities.appendString(text, message)
+        else:
+            text = self._script.utilities.appendString(message, text)
 
         if len(text.strip()):
             voice = self._script.speechGenerator.voice(string=text)
@@ -912,6 +917,9 @@ class Chat:
         - event: the accessible event being examined
         """
 
+        if event.source.getRole() != pyatspi.ROLE_TEXT:
+            return False
+
         lastKey, mods = self._script.utilities.lastKeyAndModifiers()
         if lastKey == "Tab" and event.any_data and event.any_data != "\t":
             return True
