@@ -47,6 +47,16 @@ class Script(web.Script):
 
         self.presentIfInactive = False
 
+        # Chromium doesn't always emit parent-change events. While we don't listen
+        # for those ourselves, those events tell AT-SPI2 to update the cached parent.
+        # If we cache the parent, then we can wind up getting stuck. Example page is
+        # the Github issue page when the issue search field gets focus.
+        app.setCacheMask(pyatspi.cache.DEFAULT ^
+                         pyatspi.cache.CHILDREN ^
+                         pyatspi.cache.PARENT ^
+                         pyatspi.cache.NAME ^
+                         pyatspi.cache.DESCRIPTION)
+
     def getBrailleGenerator(self):
         """Returns the braille generator for this script."""
 
@@ -155,20 +165,20 @@ class Script(web.Script):
         debug.println(debug.LEVEL_INFO, msg, True)
         default.Script.onCheckedChanged(self, event)
 
-    def onChildrenChanged(self, event):
-        """Callback for object:children-changed accessibility events."""
+    def onChildrenAdded(self, event):
+        """Callback for object:children-changed:add accessibility events."""
 
         if self.utilities.isStaticTextLeaf(event.any_data):
             msg = "CHROMIUM: Ignoring because child is static text leaf"
             debug.println(debug.LEVEL_INFO, msg, True)
             return
 
-        if super().onChildrenChanged(event):
+        if super().onChildrenAdded(event):
             return
 
         msg = "CHROMIUM: Passing along event to default script"
         debug.println(debug.LEVEL_INFO, msg, True)
-        default.Script.onChildrenChanged(self, event)
+        default.Script.onChildrenAdded(self, event)
 
     def onDocumentLoadComplete(self, event):
         """Callback for document:load-complete accessibility events."""
