@@ -131,15 +131,15 @@ class Script(web.Script):
         debug.println(debug.LEVEL_INFO, msg, True)
         default.Script.onCheckedChanged(self, event)
 
-    def onChildrenChanged(self, event):
-        """Callback for object:children-changed accessibility events."""
+    def onChildrenAdded(self, event):
+        """Callback for object:children-changed:add accessibility events."""
 
-        if super().onChildrenChanged(event):
+        if super().onChildrenAdded(event):
             return
 
         msg = "GECKO: Passing along event to default script"
         debug.println(debug.LEVEL_INFO, msg, True)
-        default.Script.onChildrenChanged(self, event)
+        default.Script.onChildrenAdded(self, event)
 
     def onDocumentLoadComplete(self, event):
         """Callback for document:load-complete accessibility events."""
